unit MkLocal;

{ MakeLocal for Delphi 32 (C) 2000, by Paul TOTH <tothpaul@free.fr>
  http://tothpaul.free.fr

  this component let you make a local copy of a network application.
  - the original version still available for update (no lock)
  - resources are loaded from local copy instead of network.

  v1.1 -AutoDate by Cyrille HUGOT <cyrille.hugot@futurtelecom.com>.

  v1.2 - by Pierre Y. <mrstone@wanadoo.fr>. on 08/08/2001
        - AutoStartDir : Lauches the local copy using the InstallPath for
                         default directory
        - CheckDates : Do not copy if the local one as the same Date as the
                        remote one (permits to always starts from the remote
                        computer because MakeLocal doesn't copy the file)
        - OnInstallAborted Event : fired when then installation has been aborted
                                    due to uptodate file or other reasons :)
}

{
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  FileCtrl,ShellAPI;

type
  TMakeLocal = class(TComponent)
  private
   fAutoInstall:boolean;
   fAutoDate : boolean;
   fAutoStartDir : boolean;
   fCheckDates : boolean;
   fInstallPath:string;
   fDirectories:TStringList;
   EOnInstall  :TNotifyEvent;
   EOnInstallAbort : TNotifyEvent;
  protected
   procedure SetDirectories(Value:TStringList);
  public
   Constructor Create(AOwner:TComponent); override;
   Destructor Destroy; override;
   Procedure Loaded; override;
   Procedure Install;
  published
   property AutoInstall:boolean        read fAutoInstall  write fAutoInstall;
   property AutoDate:boolean           read fAutoDate     write fAutoDate;
   property AutoStartDirectory:boolean read fAutoStartDir write fAutoStartDir;
   property CheckDates:boolean         read fCheckDates   write fCheckDates;
   property InstallPath:string         read fInstallPath  write fInstallPath;
   property Directories:TStringList    read fDirectories  write SetDirectories;
   property OnInstall  :TNotifyEvent   read EOnInstall    write EOnInstall;
   property OnInstallAborted : TNotifyEvent read EOnInstallAbort write EONInstallAbort;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MySoft', [TMakeLocal]);
end;

Constructor TMakeLocal.Create(AOwner:TComponent);
 begin
  inherited;
  fDirectories:=TStringList.Create;
 end;

Destructor TMakeLocal.Destroy;
 begin
  fDirectories.Free;
  inherited;
 end;

Procedure TMakeLocal.Loaded;
 begin
  inherited;
  if AutoInstall then Install;
 end;

Procedure TMakeLocal.SetDirectories(Value:TStringList);
begin
    fDirectories.Assign(Value);
end;

Procedure TMakeLocal.Install;
var
    i:integer;
    s:string;
    fs:TFileStream;
    ss:TFileStream;
    Age : integer;
begin
    if csDesigning in ComponentState then
        Exit;

    s:=Uppercase(fInstallPath+ExtractFileName(Application.ExeName));

    if s=Uppercase(Application.ExeName) then
    begin
        if Assigned(EOnInstallAbort) then
            EOnInstallAbort(Self);

        Exit; // Done
    end;

    ForceDirectories(fInstallPath);

    for i:=0 to fDirectories.count-1 do
        ForceDirectories(fDirectories[i]);

    try
        try
            Age := FileAge(Application.ExeName);

            if fCheckDates then
                if FileAge(fInstallPath+ExtractFileName(Application.ExeName)) >= Age then
                begin
                    if Assigned(EOnInstallAbort) then
                        EOnInstallAbort(Self);

                    Exit;       //Attention ce exit l xcute quand mme ce qui est dans le finally...
                                //donc on lance la copie locale mme sans avoir copi le fichier (puisqu'il est  jour !)
                end;

            fs:=TFileStream.Create(s,fmOpenWrite or fmCreate);
            ss:=TFileStream.Create(Application.ExeName,fmOpenRead or fmShareDenyNone);

            try
                fs.CopyFrom(ss,ss.Size);

                //Sinon le test de CheckDates n'a aucun sens pour le prochain lancement
                if (not fAutoDate) or fCheckDates then
                    FileSetDate(fs.Handle, Age);
            finally
                ss.Free;
                fs.Free;
            end;

            if Assigned(EOnInstall) then
                EOnInstall(Self);
        finally
            if not fAutoStartDir then
                ShellExecute(0,nil,pchar(s),nil,nil,cmdshow)
            else
                ShellExecute(0,nil,PChar(s),nil,PChar(fInstallPath),CmdShow);

            Halt;
        end;
    except
    end;
end;

end.

